"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
__all__ = ["FontCompare", "GlyhCompare","GlyphConsistency", \
"BitmapHandler","main_ui","mockify","data"]



#Test the math table

import fontforge

font=fontforge.font()
math = font.math

if math.exists():
  raise ValueError("Thinks there is a math table in an empty font")
math.clear()

math.ScriptPercentScaleDown = 3
math.SubscriptBaselineDropMin = 6
if not math.exists():
  raise ValueError("Thinks there isn't a math table after we added one")
if ( math.ScriptPercentScaleDown!=3 or math.SubscriptBaselineDropMin != 6) :
  raise ValueError("Assignment failed")

# Test some default values for math constants
math.clear()

# Create a glyph for the 'x' letter, so that math->AccentBaseHeight is nonzero.
accentBaseHeight = font.em * .75
g = font.createChar(0x78)
p = g.glyphPen();
p.moveTo(0, 0);
p.lineTo(accentBaseHeight, 0);
p.lineTo(accentBaseHeight, accentBaseHeight);
p.lineTo(0, accentBaseHeight)
p.closePath();

# Verify whether some glyph constants are unspecified by default.
if ( g.texheight != fontforge.unspecifiedMathValue or
     g.texdepth != fontforge.unspecifiedMathValue or
     g.topaccent != fontforge.unspecifiedMathValue or
     g.italicCorrection != fontforge.unspecifiedMathValue ):
  raise ValueError("Unexpected default value for a glyph constant")

# Create a glyph for the '+' letter, so that math->AxisHeight is nonzero.
axisHeight = font.em * .3
g = font.createChar(0x2B)
p = g.glyphPen();
p.moveTo(0, 0);
p.lineTo(2*axisHeight, 0);
p.lineTo(2*axisHeight, 2*axisHeight);
p.lineTo(0, 2*axisHeight)
p.closePath();

if ( math.ScriptPercentScaleDown != 80 or
     math.ScriptScriptPercentScaleDown != 60 or
     math.DelimitedSubFormulaMinHeight != int(font.em*1.5) or
     math.AxisHeight != int(axisHeight) or
     math.SubscriptTopMax != int(accentBaseHeight) or
     math.SuperscriptBottomMin != int(accentBaseHeight) or
     math.SubSuperscriptGapMin != int(4*font.uwidth) or
     math.SuperscriptBottomMaxWithSubscript != int(accentBaseHeight) or
     math.SpaceAfterScript != int(font.em/24) or
     math.StackGapMin != int(3*font.uwidth) or
     math.StackDisplayStyleGapMin != int(7*font.uwidth) or
     math.FractionNumeratorGapMin != int(font.uwidth) or
     math.FractionNumeratorDisplayStyleGapMin != int(3*font.uwidth) or
     math.FractionRuleThickness != int(font.uwidth) or
     math.FractionDenominatorGapMin != int(font.uwidth) or
     math.OverbarVerticalGap != int(3*font.uwidth) or
     math.OverbarRuleThickness != int(font.uwidth) or
     math.OverbarExtraAscender != int(font.uwidth) or
     math.UnderbarVerticalGap != int(3*font.uwidth) or
     math.UnderbarRuleThickness != int(font.uwidth) or
     math.UnderbarExtraDescender != int(font.uwidth) or
     math.RadicalVerticalGap != int(font.uwidth) or
     math.RadicalDisplayStyleVerticalGap != int(font.uwidth+.25*accentBaseHeight) or
     math.RadicalRuleThickness != int(font.uwidth) or
     math.RadicalRuleThickness != int(font.uwidth) or
     math.RadicalExtraAscender != int(font.uwidth) or
     math.RadicalKernBeforeDegree != int(5*font.em/18) or
     math.RadicalKernAfterDegree != -int(10*font.em/18) or
     math.RadicalDegreeBottomRaisePercent != 60 ):
  raise ValueError("Unexpected default value for one math constant")

math.clear()
if math.exists():
  raise ValueError("Thinks there is a math table in an empty font")

a = font.createChar(65)
c = font.createChar(67)
a.horizontalVariants = "B C D"
a.horizontalComponents = (("a",),("b",1,20,20,200),c)
a.horizontalComponentItalicCorrection = 10

a.verticalVariants = "B.v C.v D.v"
a.verticalComponents = (("a",),("b",1,20,20,200),c)
a.verticalComponentItalicCorrection = 20

if a.horizontalVariants != "B C D" or a.horizontalComponentItalicCorrection!=10:
  raise ValueError("Failed to set some glyph horizontal variant/component")
if a.verticalVariants != "B.v C.v D.v" or a.verticalComponentItalicCorrection!=20:
  raise ValueError("Failed to set some glyph vertical variant/component")

# non-regression test for a bad allocation in ParseComponentTuple
font[65].horizontalComponents = (('a',))
font[65].verticalComponents = (('a',))

#print a.verticalComponents
#print a.mathKern.topLeft

#a.mathKern.topLeft = ((1,2),(2,3))
#print a.mathKern.topLeft
#print a.mathKern.topRight



#Needs: fonts/nuvo-medium-woff-demo.woff

import os, sys, shutil, tempfile, fontforge

results = tempfile.mkdtemp('.tmp','fontforge-test-')

woff=fontforge.open(sys.argv[1])
if woff.woffMajor!=7:
  raise ValueError("Wrong return from woffMajor")
if woff.woffMinor!=504:
  raise ValueError("Wrong return from woffMinor")
metadata_len = len(woff.woffMetadata)
if metadata_len!=959 and metadata_len!=957: # Different lengths found in different versions
  raise ValueError("Wrong return from woffMetadata")

woff.woffMajor = 8

woff.generate(os.path.join(results, "Foo.woff"))

wofftest=fontforge.open(os.path.join(results, "Foo.woff"))
if wofftest.woffMajor!=8 or wofftest.woffMinor!=504:
  raise ValueError("Wrong return from woffMajor woffMinor after saving")
if len(wofftest.woffMetadata)!=metadata_len:
  raise ValueError("Wrong return from woffMetadata after saving")

shutil.rmtree(results)



"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
import os
from fc.BitmapHandler import BitmapCompare
import shutil
import pkg_resources

class DocCompare:
    def basicCompare(self,testpath, mockfont, fontsize):
        docpath = pkg_resources.resource_filename("fc","docs/doc.txt")
        bashcommand = "hb-view --output-format=\"png\" --output-file=\"/var/tmp/test.png\" --font-size="+str(fontsize)+" --text-file=\""
        bashcommand+=docpath+"\" "+"\""+testpath+"\""
        os.system(str(bashcommand))
        print bashcommand
        thefile = pkg_resources.resource_filename("fc",mockfont.highresdocfile)
        shutil.copy(thefile,"/var/tmp/standard.png")
        cm = BitmapCompare()
        score = cm.basicCompare("/var/tmp/test.png",\
        "/var/tmp/standard.png",1000,0,True)



"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
#!/usr/bin/env python
from distutils.core import setup
from setuptools import setup 
setup(name='Font Compare',
      version='1.0',
      description='FontForge based utility for aesthetic testing  of fonts',
      author='Mayank Jha',
      author_email='mayank25080562@gmail.com',
      url='https://github.com/mjnovice/FontCompare',
      include_package_data=True,
      packages=['fc'],
      package_data={'fc/data': ['data/*.mcy','data/*.bmp','data/*.png'],
      'fc/docs': ['docs/*.txt']},
      scripts = ['fontcompare'],
      install_requires=[
      "Pillow >=2.0",
      ],
    )



#Needs: fonts/Zapfino-4.0d3.dfont

import os, sys, shutil, tempfile, fontforge

results = tempfile.mkdtemp('.tmp','fontforge-test-')

zapfino = fontforge.open(sys.argv[1])
zapfino.generate(os.path.join(results, "Zapfino.ttf"),flags=("apple",))
zapfino.close()
#We used to get PointCount errors in the above.
zapfino = fontforge.open(os.path.join(results, "Zapfino.ttf"))
zapfino.close()
print("...Handled Zapfino with control points at points")

shutil.rmtree(results)



# test some ligatures and fractions close to U0000. Avoid high values such as
# UFxxx since the Unicode chart may add new unicode values later.

import sys, fontforge

print("Get Table Array Totals.")
lt = fontforge.ucLigChartGetCnt();
vt = fontforge.ucVulChartGetCnt();
ot = fontforge.ucOFracChartGetCnt();
ft = fontforge.ucFracChartGetCnt();
if lt < 50 or lt > 2000 or vt < 10 or vt > 2000 or ot < 10 or vt > 2000:
  raise ValueError("Expected totals approximately closer to 500, 50, 20.")
if vt + ot != ft:
  raise ValueError("Expected totals to add up.")

print("Test valid (and invalid) ligatures.")
l1 = fontforge.ucLigChartGetLoc(306)
l2 = fontforge.ucLigChartGetLoc(0x133)
l3 = fontforge.ucLigChartGetLoc(0x153)
l4 = fontforge.ucLigChartGetLoc(0x4b5)
l5 = fontforge.ucLigChartGetLoc(0x135)
if l1 != 0 or l2 != 1 or l3 != 3 or l4 != 7 or l5 != -1:
  raise ValueError("Expected Ligature table array 'n' values of 0,1,3,7,-1.")

print("Return some ligatures from built-in table.")
l1 = fontforge.ucLigChartGetNxt(0)
l2 = fontforge.ucLigChartGetNxt(1)
l3 = fontforge.ucLigChartGetNxt(3)
l4 = fontforge.ucLigChartGetNxt(7)
l5 = fontforge.ucLigChartGetNxt(lt) # table starts at 0, not 1 (last value is at [lt-1])
if l1 != 306 or l2 != 307 or l3 != 339 or l4 != 1205 or l5 != -1:
  raise ValueError("Expected Ligatures values from table of: 306,307,339,1205,-1.")

print("Test valid (and invalid) vulgar fractions.")
v1 = fontforge.ucVulChartGetLoc(0xbd)
v2 = fontforge.ucVulChartGetLoc(0x132)
if v1 != 1 or v2 != -1:
  raise ValueError("Expected Vulgar Fraction table array 'n' values of: 1,-1.")

print("Verify uint32 code works.")
o1 = fontforge.ucOFracChartGetNxt(ot-1)
if o1 <= 69000:
  raise ValueError("Expected last Other Fraction of 0x10e7e or larger.")

print("Test boolean found==1, not_found==0.")
b1 = fontforge.IsLigature(306)
b2 = fontforge.IsLigature(0x135)
b3 = fontforge.IsVulgarFraction(0xbd)
b4 = fontforge.IsVulgarFraction(0x132)
b5 = fontforge.IsFraction(0xbd)
b6 = fontforge.IsFraction(0x132)
if b1 != 1 or b2 != 0 or b3 != 1 or b4 != 0 or b5 != 1 or b6 != 0:
  raise ValueError("Expected boolean values of 1,0,1,0,1,0.")

print("Test valid (and invalid) vulgar fraction alt expansions.")
e1 = fontforge.ucVulChartGetLoc(0xbc)
e2 = fontforge.ucVulChartGetLoc(0x2152)
e3 = fontforge.ucVulChartGetAltCnt(e1)
e4 = fontforge.ucVulChartGetAltCnt(e2)
e5 = fontforge.ucVulChartGetAltCnt(1000)
e6 = fontforge.ucVulChartGetAltVal(e1,0)
e7 = fontforge.ucVulChartGetAltVal(e1,2)
e8 = fontforge.ucVulChartGetAltVal(e2,2)
e9 = fontforge.ucVulChartGetAltVal(e2,3)
if e1 < 0 or e2 < 0 or e3 != 3 or e4 != 4 or e5 != -1 or e6 != 49 or e7 != 52 or e8 != 49 or e9 != 48:
  print("Returned %s %s %s %s %s %s %s %s %s" % (e1,e2,e3,e4,e5,e6,e7,e8,e9))
  raise ValueError("Expected alt_expansion_count results of {0,5,3,4,-1,49,52,49,48}.")

print("Test ligature alt expansions.")
f1 = fontforge.ucLigChartGetLoc(0xfdfa)
f2 = fontforge.ucLigChartGetLoc(0xfdfb)
f3 = fontforge.ucLigChartGetAltCnt(f1)
f4 = fontforge.ucLigChartGetAltCnt(f2)
f5 = fontforge.ucLigChartGetAltVal(f1,0)
f6 = fontforge.ucLigChartGetAltVal(f1,1)
f7 = fontforge.ucLigChartGetAltVal(f1,2)
f8 = fontforge.ucLigChartGetAltVal(f1,3)
f9 = fontforge.ucLigChartGetAltVal(f1,20)
f10 = fontforge.ucLigChartGetAltVal(f2,3)
f11 = fontforge.ucLigChartGetAltVal(f2,4)
f12 = fontforge.ucLigChartUGetAltCnt(0x153)
f13 = fontforge.ucLigChartUGetAltCnt(0x4d5)
f14 = fontforge.ucLigChartUGetAltVal(0x4d5,0)
f15 = fontforge.ucLigChartUGetAltVal(0x4d5,1)
f16 = fontforge.ucLigChartUGetAltCnt(0x00011176)
f17 = fontforge.ucLigChartUGetAltVal(0x00011176,0)
if f1 < 0 or f2 < 0 or f3 != 18 or f4 != 7 or f5 != 0x635 or f6 != 0x644 or f7 != 0x649 or f8 != 0x20 or f9 != -1 or f10 != 0x62c or f11 != 0x644 or f12 != 2 or f13 != 2 or f14 != 0x430 or f15 != 0x435 or f16 != 0 or f17 != -1:
  print("Returned %s %s %s %s %s %s %s %s %s" % (f1,f2,f3,f4,f5,f6,f7,f8,f9))
  print("Returned %s %s %s %s %s %s %s %s" % (f10,f11,f12,f13,f14,f15,f16,f17))
  raise ValueError("Expected alt_expansion_count results of {496,497,18,7,1589,1604,1609,32,-1,1580,1604,2,2,1072,1077,0,-1}.")

print("All Tests done and valid.")



#Needs: fonts/Zapfino-4.1d6.dfont

import os, sys, shutil, tempfile, fontforge

results = tempfile.mkdtemp('.tmp','fontforge-test-')

print("This font has odd 'morx' tables, we just don't want to crash")
zapfino = fontforge.open(sys.argv[1])
zapfino.generate(os.path.join(results, "Zapfino.ttf"),flags=("apple",))
zapfino.close()

shutil.rmtree(results)



#Needs: fonts/NotoSans-Regular.ttc

import fontforge
import sys

for i, s in enumerate(('Noto Sans', 'Noto Sans UI')):
    for k in (str(i), s):
        f = fontforge.open(sys.argv[1] + '(' + k + ')')
        if f.fullname != s:
            raise ValueError('Expected fontname "' + s + '" instead of "' + f.fontname + '" for key "' + k + '"')
        f.close()



"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
Contains methods for making comparisions using a host of tests.
They are also for modifying a glyph by altering its
serif, stroke , stem thickness, size, italic angle etc.
and then later producing scores by doing bitmap comparision
"""
from fc.BitmapHandler import BitmapCompare
from fc.mockify import MockFont
import shutil
import pkg_resources
thefile = pkg_resources.resource_filename("fc","data/masterspritebold.bmp")
shutil.copy(thefile,"/var/tmp/tmpb.bmp")
thefile = pkg_resources.resource_filename("fc","data/masterspritenormal.bmp")
shutil.copy(thefile,"/var/tmp/tmpn.bmp")
thefile = pkg_resources.resource_filename("fc","data/masterspriteitalic.bmp")
shutil.copy(thefile,"/var/tmp/tmpi.bmp")
class FontCompare(object):

    def font_basiccompare(self, Testfont, mockfont):
        final=list()
        
        mx=max(Testfont.ascent,mockfont.ascent)
        mn=min(Testfont.ascent,mockfont.ascent)
        score1 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Ascent Score: ",score1))
        
        mx=max(Testfont.descent,mockfont.descent)
        mn=min(Testfont.descent,mockfont.descent)
        score2 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Descent Score: ",score2))
        
        mx=max(Testfont.capHeight,mockfont.capHeight)
        mn=min(Testfont.capHeight,mockfont.capHeight)
        score3 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Cap Height: ",score3))
        
        mx=max(Testfont.strokewidth,mockfont.strokewidth)
        mn=min(Testfont.strokewidth,mockfont.strokewidth)
        score4 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Stroke Width Score: ",score4))
        
        mx=max(Testfont.upos,mockfont.upos)
        mn=min(Testfont.upos,mockfont.upos)
        score5 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Underline Position Score: ",score5))
        
        mx=max(Testfont.uwidth,mockfont.uwidth)
        mn=min(Testfont.uwidth,mockfont.uwidth)
        score6 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("Underline Width Score: ",score6))
        
        mx=max(Testfont.xHeight,mockfont.xHeight)
        mn=min(Testfont.xHeight,mockfont.xHeight)
        score7 = int((1/float(abs(mx-mn)))*10) if (mx-mn)!=0 else 10
        final.append(("x Height Score: ",score7))
        
        test=Testfont.weight
        standard=mockfont.weight
        score8 = 10 if test==standard else 0
        final.append(("PostScript weight string",score8))
        
        score=score1+score2+score3+score4+score5+score6+score7+score8;
        final.append(("Average Basic Score: ",score/8.0))
        Testfont.close()
        return final

    def font_facecompare(self, Testfont, mockfont, glyphRange, \
    resolution, ptsize, pixeldepth, fonttype):
        spritepath = "/var/tmp/tmpn.bmp"
        if fonttype is "bold":
            spritepath = "/var/tmp/tmpb.bmp"
            for i in range (glyphRange[0],glyphRange[1]):
                if i in Testfont:
                    Testfont[i].changeWeight(50,"auto",0,0,"auto")
        if fonttype is "italic":
            Testfont.selection.all()
            Testfont = Testfont.italicize(-13)
            spritepath = "/var/tmp/tmpi.bmp"
        scores = list()
        comparator = BitmapCompare()
        pixelsize = (resolution*ptsize)/72
        print spritepath
        for i in range (glyphRange[0],glyphRange[1]):
            if i in Testfont:
                Testfont[i].export("/var/tmp/tmp.bmp",pixelsize,1)
                Testfont[i].export("tmp.bmp",pixelsize,1)
                glyphscore = comparator.basicCompare("/var/tmp/tmp.bmp",\
                spritepath,pixelsize,(i-glyphRange[0])*pixelsize)
                glyphscore*=100
            else:
                glyphscore=0
            scores.append((i,round(glyphscore)))
        Testfont.close()
        return scores



# -*- coding: utf-8 -*-
#Needs: fonts/DejaVuSerif.sfd

import sys, fontforge

plain = 'Comment without em-dash'
uni = u'Comment with — em dash'

f=fontforge.open(sys.argv[1])

g=f['dcroat']
if g.comment != "":
    raise ValueError("Expected comment to start as empty string")

g.comment = plain
if g.comment != plain:
    raise ValueError("Unable to assign and read plain ASCII comment")

g.comment = uni
if g.comment != uni:
    raise ValueError("Unable to assign and read plain ASCII comment")

lname = '\'=>\' Single Substitution'

# These are each string conversion tests
flags = ('right_to_left', 'ignore_bases')
fealist = (("liga",(("latn",("dflt")),)),)

f.addLookup(lname, 'gsub_single', flags, fealist)
f.addLookupSubtable(lname, lname)

g.addPosSub(lname,'LIG')
tup = g.getPosSub(lname)
if not (tup[0][0] == lname and tup[0][2] == 'LIG'):
    raise ValueError("Unable to assign and read plain ASCII PosSub")

dcroat_tti = b"@2\x1e\x04\x10\x1d\x03!\x00\x19\x13z\x1b\x11!\x94\r'\x94\x07\x15z\x17q\x00z\x07`\r\x8c\x02\x12$\x16,\x1d\x14\x10\x03\x03'\x1a\x01,\x18\x005$\x1a\n**\x10\xf4\xec\xf4<\xe42\xfc\x17<\xe4\x1191\x00/\xec\xe4\xec\xfc\xec\x10\xee\x10\xee\xd5<\xee2\x11\x12\x17990@\t\x00+\x10+\x7f+\xb0+\x04\x01]"

if g.ttinstrs != dcroat_tti:
    raise ValueError("Unexpected ttinstrs in glyph dcroat")

tbaseline = (("hang","ideo","romn"),
             (("cyrl","romn",(1405,-288,0),()),
              ("grek","romn",(1405,-288,0),()),
              ("latn","romn",(1405,-288,0),
               (("dflt",-576,1913,
                  (("NoAc",-576,1482), ("ENG ",-576,1482))
               ),)
              )
             )
            )

if f.horizontalBaseline != None:
    raise ValueError("Unexpected horizontalBaseline")
f.horizontalBaseline = tbaseline
if f.horizontalBaseline != tbaseline:
    raise ValueError("Unable to assign and read font baseline")

vrs = (f['a'], f['b'], f['c'])
#vrs = ('a', 'b', 'c')
if g.verticalVariants != None:
    raise ValueError("Unexpected verticalVariants")
g.verticalVariants = vrs
if g.verticalVariants != 'a b c':
    raise ValueError("Unable to assign and read glyph verticalVariants")



# scriptFromUnicode tests…

import sys, fontforge

assert fontforge.scriptFromUnicode(ord('ᜈ')) == "tagl"
assert fontforge.scriptFromUnicode(ord('Q')) == "latn"
assert fontforge.scriptFromUnicode(ord('θ')) == "grek"
assert fontforge.scriptFromUnicode(ord('Щ')) == "cyrl"
assert fontforge.scriptFromUnicode(ord('ツ')) == "kana"
assert fontforge.scriptFromUnicode(ord('ﬡ')) == "hebr"

try:
    fontforge.scriptFromUnicode("derp")
    assert False
except TypeError:
    pass



import sys
import fontforge

font = fontforge.open(sys.argv[1])

font.addLookup("'calt' Reverse chaining test", "gsub_reversecchain", (),
               (("calt", (("DFLT", ("DFLT",)), ("latn", ("dflt",)))),))

font.addContextualSubtable("'calt' Reverse chaining test", "reverse subtable1",
                           "reversecoverage",
                           "| [A B C D] => [a b c d] | [l m n o]")
# This test succeeds if things don't crash.
font.save("Caliban-reversechain.sfd")



#!/usr/bin/env python3

from PIL import Image
import sys
import textwrap

TEMPLATE = """/* Copyright (C) 2000-2012 by George Williams 
 * Image replaced in 2019 by @ctrlcctrlv (Fred Brennan) */
/*
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:

 * Redistributions of source code must retain the above copyright notice, this
 * list of conditions and the following disclaimer.

 * Redistributions in binary form must reproduce the above copyright notice,
 * this list of conditions and the following disclaimer in the documentation
 * and/or other materials provided with the distribution.

 * The name of the author may not be used to endorse or promote products
 * derived from this software without specific prior written permission.

 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR IMPLIED
 * WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO
 * EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
 * SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS;
 * OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY,
 * WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR
 * OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF
 * ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

#include <fontforge-config.h>

#include "gimage.h"

static uint8 splashimage0_data[] = {{
{data}
}};

static struct _GImage splashimage0_base = {{
    it_true,
    0,
    {width},
    {height},
    {bytes_per_line},
    (uint8 *) splashimage0_data,
    NULL,
    0xffffffff
}};

GImage splashimage = {{ 0, {{ &splashimage0_base }}, NULL }};
"""

if __name__ == "__main__":
    if len(sys.argv) != 2:
        print("Usage: {} input.png".format(sys.argv[0]))
        sys.exit(-1)

    img = Image.open(sys.argv[1]).convert("RGB")
    data = ", ".join(
        "0x{:02x}, 0x{:02x}, 0x{:02x}, 0xff".format(b, g, r)
        for r, g, b in img.getdata()
    )
    wrapper = textwrap.TextWrapper(
        width=79,
        break_long_words=False,
    )
    data = "\n".join(wrapper.wrap(data))

    with open("splashimage.c", "w") as fp:
        fp.write(
            TEMPLATE.format(
                data=data,
                width=img.width,
                height=img.height,
                bytes_per_line=img.width * 4,
            )
        )



import sys, fontforge

font = fontforge.font()
feat = sys.argv[1]

A = font.createChar(ord("A"), "A")
pen = A.glyphPen()
pen.moveTo((100,100))
pen.lineTo((100,200))
pen.lineTo((200,200))
pen.lineTo((200,100))
pen.closePath()

B = font.createChar(ord("B"), "B")
pen = B.glyphPen()
pen.moveTo((100,100))
pen.lineTo((100,200))
pen.lineTo((200,200))
pen.lineTo((200,100))
pen.closePath()

font.mergeFeature(feat)

expected = (0, 0, 100, 0, 0, 0, 0, 0)
assert A.getPosSub(font.getLookupSubtables("ltr")[0])[0][3:] == expected
assert A.getPosSub(font.getLookupSubtables("rtl")[0])[0][3:] == expected

font.generateFeatureFile("test1011.result.fea")

with open(feat) as expected_file:
    with open("test1011.result.fea") as result_file:
        expected = expected_file.readlines()
        result = result_file.readlines()
        if result != expected:
            from difflib import Differ
            d = Differ()
            sys.stderr.writelines(list(d.compare(result, expected)))
            sys.exit(1)
sys.exit(0)



'''
Copyright (C) 2020 by Jeremy Tan

Distributed under the original FontForge BSD 3-clause license.
Based on the docutils table code, which is released into the public domain.

-------------------------------------------------------------------------------

Adds (HTML only) support for a flex-grid directive that allows writing
out elements in a generic grid fashion.

Example:

.. flex-grid::
  :class: custom_class

  * - Row 1 Col 1
    - Row 1 Col 2
    - Row 1 col 3
  * :flex-widths: 1 2
    - Row 2 Col 1
    - Row 2 Col 2

The syntax is very similar to the docutils list-table directive. However the
number of columns per row does *not* need to be homogenous.

The flex-widths parameter is specified on a per-row basis. It should be a list
of numbers whose length is equal to the number of columns in that row. It
represents the ratio of space taken by that column. For example, with two
columns and a flex-widths of '1 2', the proportion of space taken is 1:2
between the first and second columns. A value of 0 disables flexing that
column.
'''

from docutils import nodes, statemachine
from docutils.utils import SystemMessagePropagation
from docutils.parsers.rst import Directive
from docutils.parsers.rst import directives


class flex_grid(nodes.General, nodes.Element): pass
class flex_row(nodes.Part, nodes.Element): pass
class flex_col(nodes.Part, nodes.Element): pass


class FlexGrid(Directive):

    final_argument_whitespace = True
    option_spec = {'class': directives.class_option, 'name': directives.unchanged}
    has_content = True
    valid_flexes = set((0, 1, 2, 3, 4, 5))

    def run(self):
        if not self.content:
            error = self.state_machine.reporter.error(
                'The "%s" directive is empty; content required.' % self.name,
                nodes.literal_block(self.block_text, self.block_text),
                line=self.lineno)
            return [error]

        node = nodes.Element()          # anonymous container for parsing
        self.state.nested_parse(self.content, self.content_offset, node)

        try:
            grid_data = self.get_grid(node)
        except SystemMessagePropagation as detail:
            return [detail.args[0]]

        grid_node = self.build_grid_node(grid_data)
        return [grid_node]

    def get_grid(self, node):
        if len(node) != 1 or not isinstance(node[0], nodes.bullet_list):
            error = self.state_machine.reporter.error(
                'Error parsing content block for the "%s" directive: '
                'exactly one bullet list expected.' % self.name,
                nodes.literal_block(self.block_text, self.block_text),
                line=self.lineno)
            raise SystemMessagePropagation(error)

        grid = []
        list_node = node[0]

        for item_index in range(len(list_node)):
            item = list_node[item_index]
            nitems = len(item)
            if not nitems or nitems > 2 or not isinstance(item[-1], nodes.bullet_list):
                error = self.state_machine.reporter.error(
                    'Error parsing content block for the "%s" directive: '
                    'two-level bullet list expected, but row %s does not '
                    'contain a second-level bullet list.'
                    % (self.name, item_index + 1), nodes.literal_block(
                        self.block_text, self.block_text), line=self.lineno)
                raise SystemMessagePropagation(error)
            elif nitems == 2 and not isinstance(item[0], nodes.field_list):
                error = self.state_machine.reporter.error(
                    'Error parsing content block for the "%s" directive: '
                    'expected options as a field list, but row %s does not '
                    'start with a field list.'
                    % (self.name, item_index + 1), nodes.literal_block(
                        self.block_text, self.block_text), line=self.lineno)
                raise SystemMessagePropagation(error)

            if nitems == 1:
                row = ({}, [col.children for col in item[0]])
            else:
                opts = {}
                for opt in item[0]:
                    key = opt[0].rawsource
                    value = opt[1].rawsource.strip()

                    if key != 'flex-widths':
                        error = self.state_machine.reporter.error(
                            'Error parsing content block for the "%s" directive: '
                            'received "%s" but only "flex-widths" is supported as '
                            'an option on row %s.'
                            % (self.name, key, item_index + 1), nodes.literal_block(
                                self.block_text, self.block_text), line=self.lineno)
                        raise SystemMessagePropagation(error)

                    try:
                        sep = ',' if ',' in value else ' '
                        fwidths = [int(x) for x in value.split(sep)]
                        if any(x not in self.valid_flexes for x in fwidths) or len(fwidths) != len(item[1]):
                            raise ValueError('invalid flex widths')
                        opts['flex-widths'] = fwidths
                    except:
                        error = self.state_machine.reporter.error(
                            'Error parsing content block for the "%s" directive: '
                            '"flex-widths" expects a list of %s widths with possible values of (1,2,3,4,5).'
                            'The input "%s" for row %s is malformed.'
                            % (self.name, len(item[1]), value, item_index + 1), nodes.literal_block(
                                self.block_text, self.block_text), line=self.lineno)
                        raise SystemMessagePropagation(error)
                row = (opts, [col.children for col in item[1]])
            grid.append(row)
        return grid

    def build_grid_node(self, grid_data):
        grid = flex_grid()
        self.add_name(grid)
        grid['classes'] += ['flex-container']
        grid['classes'] += self.options.get('class', [])
        for row_opts, row in grid_data:
            row_node = flex_row()
            row_node['classes'] += ['flex-row']
            widths = row_opts.get('flex-widths')

            for i, cell in enumerate(row):
                entry = flex_col()
                entry['classes'] += ['flex-col']
                if widths:
                    width = widths[i]
                    if width == 0:
                        entry['classes'] += ['flex-none']
                    elif width != 1:
                        entry['classes'] += ['flex-{}'.format(width)]
                entry += cell
                row_node += entry
            grid += row_node
        return grid


def visit_flex_node(self, node):
    self.body.append(self.starttag(node, 'div'))

def depart_flex_node(self, node):
    self.body.append('</div>\n')


def setup(app):
    app.add_directive('flex-grid', FlexGrid)
    app.add_node(flex_grid, html=(visit_flex_node, depart_flex_node))
    app.add_node(flex_row, html=(visit_flex_node, depart_flex_node))
    app.add_node(flex_col, html=(visit_flex_node, depart_flex_node))

    return {
        'version': '1.0.0',
        'env_version': 1,
        'parallel_read_safe': True
    }



# test some libuninameslist functions (if the library is included).

import sys, fontforge

cnt = fontforge.UnicodeBlockCountFromLib()
if cnt < 1:
  print("No NamesList library - Nothing to test.")
  sys.exit()

if cnt < 294:
  print("Pre-20170701 or older library - Nothing to test beyond here.")
  sys.exit()

ver = fontforge.UnicodeNamesListVersion()
print("Libuninameslist version is : %s" % ver)

# test Names2 functions (if we have libuninameslist version>=10.9)
if cnt < 305:
  print("Pre-20180408 or older library - Nothing to test beyond here.")
  sys.exit()

cnt = fontforge.UnicodeNames2GetCntFromLib()
print("Libuninameslist internal table has %s Names2." % cnt)
if cnt < 25:
  print("Test Names2 functions. Must be libuninameslist 10.9 or newer.")
  raise ValueError("Expected 25 or more values that have Names2.")

# none of these are -1 if library loaded
u0 = fontforge.UnicodeNames2NxtUniFromLib(0)
u1 = fontforge.UnicodeNames2NxtUniFromLib(1)
u2 = fontforge.UnicodeNames2NxtUniFromLib(2)
print("Sample of internal table has %s, %s, %s as the first 3 unicode values." % (u0, u1, u2))
if u0 < 0 or u1 < 1 or u2 < 2:
  raise ValueError("these values may be different, but expected 418,419,1801 for ver10.9.")

# We know these Names2 exist, but these 'tX' values may change if more errors found
t0 = fontforge.UnicodeNames2GetNxtFromLib(0x01A2)
t1 = fontforge.UnicodeNames2GetNxtFromLib(0xFEFF)
t2 = fontforge.UnicodeNames2GetNxtFromLib(118981)
print("Internal table[%s]=0x01A2, table[%s]=0xFEFF, table[%s]=118981 as expected Names2 values." % (t0, t1, t2))
if t0 < 0 or t1 < 0 or t2 < 0:
  raise ValueError("These Names2 must exist, so we expected these to return valid table locations.")

s0 = fontforge.UnicodeNames2FrmTabFromLib(3)
print("Example unicode from table[3]=%s" % s0)
s1 = fontforge.UnicodeNames2FromLib(65)
s2 = fontforge.UnicodeNames2FromLib(0x709)
print("There should be no Names2 for character 65, the value returned is=%s" % s1)
print("There should be a Names2 for character 0x709, the value returned is=%s" % s2)
if len(s0) < 3 or len(s2) < 3:
  raise ValueError("Expected to return a valid Names2 string.")

print("All Tests done and valid.")




#Needs: fonts/ayn+meem.init.svg

# At one point splinestroke failed if the first spline on a contour had a length of 0

import sys, fontforge

font=fontforge.font()
a = font.createChar(65)
a.importOutlines(sys.argv[1])



#Needs: fonts/StrokeTests.sfd

import sys, fontforge, psMat, math
from collections import OrderedDict

stroketests = sys.argv[1]
shortlist = ['D', 'Q', 'asciicircum']
convex = fontforge.unitShape(7).transform(psMat.scale(25))

def do_stroke_test(short, *args, **kwargs):
    font=fontforge.open(stroketests)
    if short:
        font.selection.select(*shortlist)
    else:
        font.selection.all()
    font.stroke(*args, **kwargs)
    font.close()

# Full font tests
shapes = OrderedDict()
shapes['triangle'] = fontforge.unitShape(3).transform(psMat.scale(25))
shapes['square'] = fontforge.unitShape(4).transform(psMat.scale(25))
shapes['pentagon'] = fontforge.unitShape(5).transform(psMat.scale(25))
shapes['circular'] = (25,)
shapes['calligraphic'] = (25, 2, 45)

for sn, s in shapes.items():
    print(sn)
    if isinstance(s, tuple):
        do_stroke_test(False, sn, *s)
    else:
        do_stroke_test(False, 'polygonal', s)

# Argument tests
do_stroke_test(True, 'circular', 70)
do_stroke_test(True, 'circular', 30, "butt", "bevel")
do_stroke_test(True, 'elliptical', 30, 40, math.radians(45), "butt", "bevel")
do_stroke_test(True, 'elliptical', 50, 20, math.radians(25), "butt", "miter")
do_stroke_test(True, 'elliptical', 10, 100, 0, "butt", "miterclip")
do_stroke_test(True, 'eliptical', 40, 42, math.radians(20), "butt", "nib")
do_stroke_test(True, 'caligraphic', 30, 40, math.radians(45), "butt", "bevel")
do_stroke_test(True, 'rectangular', 50, 20, math.radians(25), "nib", "miter")
do_stroke_test(True, 'rectangular', 50, 20, math.radians(25), "nib", "round")
do_stroke_test(True, 'rectangular', 10, 100, 0, "round", "miterclip")
do_stroke_test(True, 'caligraphic', 40, 42, math.radians(20), "butt", "nib")
do_stroke_test(True, 'convex', convex, math.radians(45), "butt", "bevel")
do_stroke_test(True, 'polygonal', convex, math.radians(25), "nib", "miter")
do_stroke_test(True, 'convex', convex, 0, "round", "miterclip")
do_stroke_test(True, 'polygonal', convex, math.radians(20), "butt", "nib")

do_stroke_test(True, 'calligraphic', 20, 100)
do_stroke_test(True, 'calligraphic', 20, 100, angle=math.radians(30))
do_stroke_test(True, 'calligraphic', 20, 100, join="miterclip")
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round")
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", removeinternal=True)
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", removeexternal=True)
do_stroke_test(True, 'circular', 60, join="nib", cap="nib", extrema=False, angle=45)
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", simplify=False)
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", accuracy=3)
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", removeoverlap="contour")
do_stroke_test(True, 'calligraphic', 20, 100, join="bevel", cap="round", removeoverlap="none")
do_stroke_test(True, 'calligraphic', 20, 100, join="miter", cap="round", joinlimit=4)
do_stroke_test(True, 'calligraphic', 20, 100, join="miterclip", cap="round", joinlimit=4)
do_stroke_test(True, 'calligraphic', 20, 100, join="miterclip", cap="round", joinlimit=180, jlrelative=False)
do_stroke_test(True, 'calligraphic', 20, 100, join="miterclip", cap="round", extendcap=3)
do_stroke_test(True, 'calligraphic', 20, 100, join="miterclip", cap="round", extendcap=100, ecrelative=False)
do_stroke_test(True, 'elliptical', 20, 20, join="arcs", cap="butt", joinlimit=4)
do_stroke_test(True, 'elliptical', 20, 100, join="arcs", cap="round", joinlimit=2, arcsclip="ratio")



#!/usr/bin/python
#
# Copyright 2013 Google Inc. All rights reserved.
# 
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
# 
#     http://www.apache.org/licenses/LICENSE-2.0
# 
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
# A simple script to convert svg files (generated by Illustrator plugin)
# to sfd, from which Fontforge can then create a font file.
#
# Contributors: Raph Levien (raph@google.com)

import sys
import os.path
import glob
import xml.dom.minidom
import re
import math

lastglyphnum = 0
char_num = 0x40
font_name = 'Untitled'
header_printed = False

def num_args_cmd(cmd):
  if cmd.upper() == 'C': return 6
  elif cmd.upper() in 'HV': return 1
  elif cmd.upper() == 'S': return 4
  elif cmd == 'z': return 0
  return 2

def print_one_cmd(cmd, args):
  scale = 40
  yoff = 720
  result = []
  for i in range(len(args)):
    if i & 1:
      result.append('%f' % (yoff - scale * args[i]))
    else:
      result.append('%f' % (scale * args[i]))
  result.append(cmd)
  result.append('0')  # TODO: should mark corner points
  print ' '.join(result)

def apply_rel_xy(xy, args):
  x0, y0 = xy
  result = []
  for i in range(0, len(args), 2):
    x = x0 + args[i]
    result.append(x)
    y = y0 + args[i + 1]
    result.append(y)
  return result

def path_to_sfd(path):
  # convert svg path syntax into sfd
  # written for conciseness, not efficiency
  x0, y0 = 0, 0
  fre = re.compile(r'(\-?[0-9\.]+)\s*,?\s*')
  while path.strip() != '':
    path = path.strip()
    if path[0].isalpha():
      cmd = path[0]
      path = path[1:].lstrip()
    args = []
    for i in range(num_args_cmd(cmd)):
      m = fre.match(path)
      if m is None:
        print 'no float match:', path
      args.append(float(m.group(1)))
      path = path[m.end():]
    #print cmd, args
    if cmd.upper() == 'M':
      if cmd.islower(): (x, y), args = apply_rel_xy([x, y], args)
      x0, y0 = args
      print_one_cmd('m', args)
      x, y = args[-2:]
      if cmd == 'm': cmd = 'l'
      elif cmd == 'M': cmd = 'L'
    elif cmd.upper() in 'CLVHS':
      if cmd == 'H':
        args = args + [y]
        cmd = 'L'
      elif cmd == 'h':
        args = args + [0]
        cmd = 'l'
      if cmd == 'V':
        args = [x] + args
        cmd = 'L'
      elif cmd == 'v':
        args = [0] + args
        cmd = 'l'
      if cmd.islower(): args = apply_rel_xy([x, y], args)
      if cmd.upper() == 'S':
        # smooth curveto; reflect
        args = [2 * x - xs, 2 * y - ys] + args
        cmd = 'c'
      print_one_cmd(cmd.lower(), args)
      x, y = args[-2:]
      if len(args) > 2:
        xs, ys = args[-4:-2]
    elif cmd.upper() == 'Z':
      if x != x0 or y != y0:
        print_one_cmd('l', [x0, y0])

def circle_to_sfd(cx, cy, r):
  k = 4 * (math.sqrt(2) - 1) / 3
  print_one_cmd('m', [cx, cy - r])
  print_one_cmd('c', [cx + k * r, cy - r, cx + r, cy - k * r, cx + r, cy])
  print_one_cmd('c', [cx + r, cy + k * r, cx + k * r, cy + r, cx, cy + r])
  print_one_cmd('c', [cx - k * r, cy + r, cx - r, cy + k * r, cx - r, cy])
  print_one_cmd('c', [cx - r, cy - k * r, cx - k * r, cy - r, cx, cy - r])

def conv_svg(fn, char, glyphnum = None):
  global lastglyphnum
  global header_printed
  if not header_printed:
    print_header()
  if glyphnum == None:
    glyphnum = lastglyphnum + 1
  lastglyphnum = glyphnum
  print 'StartChar:', os.path.basename(fn)[:-4]
  print 'Encoding: %d %d %d' % (char, glyphnum, char)
  print 'Width: %d' % (21 * 40)
  print 'Flags: W'
  print 'LayerCount: 2'
  print 'Fore'
  print 'SplineSet'
  doc = xml.dom.minidom.parse(fn)
  # TODO: reverse paths if fill color is white-ish (this is more code,
  # and in the meantime, we'll rely on correct path direction in FF)
  for path in doc.getElementsByTagName('path'):
    path_to_sfd(path.getAttribute('d'))
  for polygon in doc.getElementsByTagName('polygon'):
    path_to_sfd('M' + polygon.getAttribute('points') + 'z')
  for circle in doc.getElementsByTagName('circle'):
    cx = float(circle.getAttribute('cx'))
    cy = float(circle.getAttribute('cy'))
    r = float(circle.getAttribute('r'))
    circle_to_sfd(cx, cy, r)
  print 'EndSplineSet'
  print 'EndChar'

def print_header():
  global header_printed
  print '''SplineFontDB: 3.0
FontName: %s
FullName: %s
FamilyName: %s''' % (font_name, font_name, font_name)
  print '''Weight: Medium
Copyright: Copyright (C) 2011 Google Inc.
Version: 001.000
UnderlinePosition: -120
UnderlineWidth: 40
Ascent: 800
Descent: 200
LayerCount: 2
Layer: 0 0 "Back" 1
Layer: 1 0 "Fore" 0
Encoding: unicode
OS2TypoAscent: 800
OS2TypeAOffset: 0
OS2TypoDescent: -200
OS2TypoDOffset: 0
OS2WinAscent: 800
OS2WinAOffset: 0
OS2WinDescent: 200
OS2WinDOffset: 0
HheadAscent: 800
HheadAOffset: 0
HheadDescent: 200
HheadDOffset: 0
BeginChars: 57600 57600
'''
  header_printed = True

def print_footer():
  print '''EndChars
EndSplineFont'''

def parse_int(x):
  if x.startswith('0x'):
    return int(x[2:], 16)
  else:
    return int(x)

def run_file(fn):
  global char_num
  global font_name
  directory = ''
  for l in file(fn).xreadlines():
    if l.startswith('#'):
      continue
    s = l.strip().split()
    if len(s) == 0:
      continue
    if s[0] == 'dir':
      directory = s[1]
    elif s[0] == 'fontname':
      font_name = s[1]
    elif s[0] == 'unicode':
      char_num = parse_int(s[1])
    elif s[0] == 'icon':
      icon_fn = s[1]
      if not icon_fn.endswith('.svg'):
        icon_fn += '.svg'
      if len(s) > 2:
        char_num = parse_int(s[2])
      conv_svg(os.path.join(directory, icon_fn), char_num)
      char_num += 1

def main(args):
  global char_num
  for arg in args:
    if os.path.isdir(arg):
      for fn in glob.glob(arg + '/*.svg'):
        conv_svg(fn, char_num)
        char_num += 1
    elif arg.endswith('.svg'):
      conv_svg(arg, char_num)
      char_num += 1
    else:
      run_file(arg)
  print_footer()

if __name__ == '__main__':
  main(sys.argv[1:])



import fontforge

font = fontforge.font()
glyph = font.createChar(0x41)

# Any data in the foreground
pen = glyph.glyphPen()
pen.moveTo((0,0))
pen.lineTo((1,1))
pen.lineTo((1,0))
pen.closePath()
del pen

# Switch to BG
glyph.activeLayer = 0
glyph.background.is_quadratic = False
pen = glyph.glyphPen()
pen.moveTo((1,1))
pen.closePath()
del pen # crash if (auto)destroyed after font.close()...

font.generate('a.ttf', flags=('PfEd-background'))
font.close()

font = fontforge.open('a.ttf')
glyph = font[0x41]

assert len(glyph.foreground) == 1, 'Foreground contains a contour'
assert len(glyph.background) == 1, 'Background contains a contour'

fgc = fontforge.contour(True)
fgc.closed = True
fgc[:] = ((0,0), (1,1), (1,0))
# contour comparator is inexact, see PyFFContour_docompare / SSsCompare
assert glyph.foreground[0] == fgc, 'Foreground contour matches expected contour'
assert list(glyph.foreground[0]) == list(fgc), 'Foreground contour points matches expected contour points'

bgc = fontforge.contour(False)
bgc.closed = True
bgc[:] = ((1,1),)
assert glyph.background[0] == bgc, 'Background contour matches expected contour'
assert list(glyph.background[0]) == list(bgc), 'Foreground contour points matches expected contour points'

font.close()



"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from fc.BitmapHandler import CreateSpriteSheet
import shelve
import fontforge
import os
class MockFont:
    ascent=0
    descent=0
    strokewidth=0
    uwidth=0
    upos=0
    xHeight=0
    glyphs = list()
    capHeight=0
    normalglyphfile = "data/masterspritenormal.bmp"
    boldglyphfile = "data/masterspritebold.bmp"
    italicglyphfile = "data/masterspriteitalic.bmp"
    highresdocfile = "data/highres.png"
    def __init__(self,fontpath,pixelsize,glyphrange):
        font = fontforge.open(fontpath)
        self.ascent = font.ascent
        self.descent = font.descent
        self.upos = font.upos
        self.uwidth = font.uwidth
        self.xHeight = font.xHeight
        self.capHeight = font.capHeight
        self.weight = font.weight
        #normal 
        CreateSpriteSheet(pixelsize,font,glyphrange,"normal")
        font = fontforge.open(fontpath)
        #bold
        CreateSpriteSheet(pixelsize,font,glyphrange,"bold")
        font  = fontforge.open(fontpath)
        #italic
        CreateSpriteSheet(pixelsize,font,glyphrange,"italic")
        #docfile,highres
        bashcommand = "hb-view --output-format=\"png\" --output-file=\"data/highres.png\" --font-size=160 --text-file=\""
        bashcommand+="docs/doc.txt\" \""+fontpath+"\""
        os.system(str(bashcommand))

def main():
    mock_font = shelve.open("data/mockfile.mcy")
    mock_font["font"] = MockFont("../unittests/lohit.ttf",100,(0x900,0x97f))
    mock_font.close()

if  __name__ =='__main__':main()
"""
explanation--> 

"../unittests/lohit.ttf" is the path where the font file is stored
so all you need to do is change the path of the font file, for making 
your own mockfonts.

"""



#Needs: fonts/BoxesAndBounds.py

import sys, fontforge

font = fontforge.open(sys.argv[1])

expected = (63.0, 57.0, 315.0, 309.0)
assert font[0].boundingBox() == expected
assert font[1].boundingBox(layer=1) == expected
nullbb = (0.0, 0.0, 0.0, 0.0)
assert font[1].boundingBox(layer=0) == nullbb

expected = (63.0, 57.0, 315.0, 711.0)
assert font[2].boundingBox() == expected
expected = (63.0, 459.0, 315.0, 711.0)
assert font[2].boundingBox(layer=2) == expected

xmin, ymin, xmax, ymax = font[2].boundingBox()

assert (xmin, xmax) == font[2].xBoundsAtY(ymin, ymax)
assert font[2].xBoundsAtY(ymax+1) == None

assert (ymin, ymax) == font[2].yBoundsAtX(xmin, xmax)
assert font[2].yBoundsAtX(xmax+1) == None



"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
"""
    Copyright (C) 2013 Mayank Jha <mayank25080562@gmail.com>

    This library is free software; you can redistribute it and/or modify
    it under the terms of the GNU Lesser General Public License as published
    by the Free Software Foundation; either version 2.1 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU Lesser General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
"""
from PIL import Image
def white_bg_square(img):
    "return a white-background-color image having the img in exact center"
    size = (max(img.size),)*2
    layer = Image.new('1', size, 1)
    layer.paste(img, tuple(map(lambda x:(x[0]-x[1])/2, zip(size, img.size))))
    return layer

class BitmapCompare:
    def basicCompare(self,testbmp,standardsprite,size,xoffset,equal=False):
        master = Image.new('1',(size,size),0)
        img = Image.open(testbmp)
        square_one = white_bg_square(img)
        square_one.resize((size,size))
        master.paste(square_one,(0,0))
        master.save(testbmp)
        if equal:
            master = Image.new('1',(size,size),0)
            img = Image.open(standardsprite)
            square_one = white_bg_square(img)
            square_one.resize((size,size))
            master.paste(square_one,(0,0))
            master.save(standardsprite)
        sprite = Image.open(standardsprite) 
        test = Image.open(testbmp)
        count = 0
        i=0
        while i<size:
            j=0
            while j<size:
                if sprite.getpixel((j+xoffset,i)) == test.getpixel((j,i)):
                    count+=1
                j=j+1
            i=i+1
        return float(count)/(size*size)


class CreateSpriteSheet:
    def __init__(self,pixelsize,font,glyphrange,effects):
        master_width = (pixelsize * (glyphrange[1]-glyphrange[0]+1) )
        #seperate each image with lots of whitespace
        master_height = pixelsize
        oldfont = font
        print "the master image will by %d by %d" % (master_width, master_height)
        print "creating image..."
        master = Image.new(
        mode='1',
        size=(master_width, master_height),
        color=0) # fully transparent
        print "created."
        if effects == "italic":
            font.selection.all()
            font = font.italicize(-13)
        count=0
        for i in range (glyphrange[0],glyphrange[1]):
            location = pixelsize*count
            try:
                t=font[i]
                if effects == "bold":
                    font[i].changeWeight(50,"auto",0,0,"auto")
                font[i].export("temp.bmp",pixelsize,1)
                img = Image.open("temp.bmp")
                print "adding %s at %d..." % (str(i)+".bmp", location),
                square_one = white_bg_square(img)
                square_one.resize((pixelsize, pixelsize))
                master.paste(square_one,(location,0))
                print "added."
            except:
                print "ooopsy"
            count+=1
        print "done adding pics."
        print "saving mastersprite.bmp..."
        master.save('data/mastersprite'+effects+'.bmp' )
        print "saved!"
        font.close()
